#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// deno-fmt-ignore-file
const dntShim = __importStar(require("./_dnt.shims.js"));
const command_js_1 = require("./deps/deno.land/x/cliffy@v0.25.7/command/command.js");
const mod_js_1 = require("./deps/deno.land/std@0.181.0/path/mod.js");
const index_js_1 = require("./index.js");
function getPackageVersion() {
    try {
        return JSON.parse(dntShim.Deno.readTextFileSync((0, mod_js_1.resolve)(new URL(require("url").pathToFileURL(__filename).href).pathname, "../../package.json"))).version;
    }
    catch {
        return "-";
    }
}
new command_js_1.Command()
    /*
     * Main command
     */
    .name("unity-changeset")
    .version(getPackageVersion)
    .description("Find Unity changesets.")
    .example("unity-changeset 2018.4.36f1", "Get changeset of Unity 2018.4.36f1 ('6cd387d23174' will be output).")
    .arguments("<version>")
    .globalOption("--db", "Database url", { default: "https://mob-sakai.github.io/unity-changeset/db" })
    .action((options, version) => {
    (0, index_js_1.getUnityChangeset)(options.db, version)
        .then((c) => console.log(c.changeset))
        .catch(() => {
        console.error("The given version was not found.");
        dntShim.Deno.exit(1);
    });
})
    /*
     * Sub command: list.
     */
    .command("list", new command_js_1.Command()
    .description("List Unity changesets.")
    .globalOption("--db", "Database url", { default: "https://mob-sakai.github.io/unity-changeset/db" })
    .example("unity-changeset list", "List changesets.")
    .example("unity-changeset list --all --json", "List changesets of all versions in json format.")
    .example("unity-changeset list --version-only --min 2018.3 --max 2019.4", "List all versions from 2018.3 to 2019.4.")
    .example("unity-changeset list --version-only --grep '(2018.4|2019.4)'", "List all versions in 2018.4 and 2019.4.")
    .example("unity-changeset list --lts --latest-patch", "List changesets of the latest patch versions (LTS only).")
    // Search options.
    .group("Search options")
    .option("--all", "Search all changesets (alpha/beta included)", { conflicts: ["pre-release"] })
    .option("--pre-release, --beta", "Search only pre-release (alpha/beta) changesets", { conflicts: ["all"] })
    // Filter options.
    .group("Filter options")
    .option("--min <version>", "Minimum version (included)")
    .option("--max <version>", "Maximum version (included)")
    .option("--grep <regex>", "Regular expression (e.g. '20(18|19).4.*')")
    .option("--latest-lifecycle", "Only the latest lifecycle (default)")
    .option("--all-lifecycles", "All lifecycles", { conflicts: ["latest-lifecycle"] })
    .option("--lts", "Only the LTS versions", { conflicts: ["pre-release"] })
    // Group options.
    .group("Group options")
    .option("--latest-patch", "The latest patch versions only")
    .option("--oldest-patch", "The oldest patch versions in lateat lifecycle only", { conflicts: ["latest-patch"] })
    // Output options.
    .group("Output options")
    .option("--version-only, --versions", "Outputs only the version (no changesets)")
    .option("--minor-version-only, --minor-versions", "Outputs only the minor version (no changesets)", { conflicts: ["version-only"] })
    .option("--json", "Output in json format")
    .option("--pretty-json", "Output in pretty json format")
    .action((options) => {
    // Search mode.
    const searchMode = options.all
        ? index_js_1.SearchMode.All
        : options.preRelease
            ? index_js_1.SearchMode.PreRelease
            : index_js_1.SearchMode.Default;
    // Group mode.
    const groupMode = (options.latestPatch || options.minorVersionOnly)
        ? index_js_1.GroupMode.LatestPatch
        : options.oldestPatch
            ? index_js_1.GroupMode.OldestPatch
            : index_js_1.GroupMode.All;
    // Filter options.
    const filterOptions = {
        min: options.min || "",
        max: options.max || "",
        grep: options.grep || "",
        allLifecycles: (options.allLifecycles && !options.latestLifecycle)
            ? true
            : false,
        lts: options.lts || false,
    };
    // Output mode.
    const outputMode = options.versionOnly
        ? index_js_1.OutputMode.VersionOnly
        : options.minorVersionOnly
            ? index_js_1.OutputMode.MinorVersionOnly
            : index_js_1.OutputMode.Changeset;
    // Format mode.
    const formatMode = options.json
        ? index_js_1.FormatMode.Json
        : options.prettyJson
            ? index_js_1.FormatMode.PrettyJson
            : index_js_1.FormatMode.None;
    (0, index_js_1.listChangesets)(options.db, searchMode, filterOptions, groupMode, outputMode, formatMode)
        .then((result) => console.log(result));
}))
    /*
     * Run with arguments.
     */
    .parse(dntShim.Deno.args);
