"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TooManyArgumentsError = exports.MissingArgumentError = exports.MissingArgumentsError = exports.NoArgumentsAllowedError = exports.UnknownCommandError = exports.UnknownCompletionCommandError = exports.CommandExecutableNotFoundError = exports.DefaultCommandNotFoundError = exports.UnexpectedVariadicEnvVarValueError = exports.UnexpectedOptionalEnvVarValueError = exports.TooManyEnvVarValuesError = exports.MissingRequiredEnvVarError = exports.DuplicateEnvVarError = exports.DuplicateExampleError = exports.DuplicateCompletionError = exports.DuplicateTypeError = exports.CommandNotFoundError = exports.DuplicateCommandAliasError = exports.DuplicateCommandNameError = exports.MissingCommandNameError = exports.DuplicateOptionNameError = exports.ValidationError = exports.CommandError = void 0;
const _utils_js_1 = require("./_utils.js");
const _utils_js_2 = require("../flags/_utils.js");
class CommandError extends Error {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, CommandError.prototype);
    }
}
exports.CommandError = CommandError;
class ValidationError extends CommandError {
    constructor(message, { exitCode } = {}) {
        super(message);
        Object.defineProperty(this, "exitCode", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "cmd", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.setPrototypeOf(this, ValidationError.prototype);
        this.exitCode = exitCode ?? 1;
    }
}
exports.ValidationError = ValidationError;
class DuplicateOptionNameError extends CommandError {
    constructor(name) {
        super(`Option with name "${(0, _utils_js_2.getFlag)(name)}" already exists.`);
        Object.setPrototypeOf(this, DuplicateOptionNameError.prototype);
    }
}
exports.DuplicateOptionNameError = DuplicateOptionNameError;
class MissingCommandNameError extends CommandError {
    constructor() {
        super("Missing command name.");
        Object.setPrototypeOf(this, MissingCommandNameError.prototype);
    }
}
exports.MissingCommandNameError = MissingCommandNameError;
class DuplicateCommandNameError extends CommandError {
    constructor(name) {
        super(`Duplicate command name "${name}".`);
        Object.setPrototypeOf(this, DuplicateCommandNameError.prototype);
    }
}
exports.DuplicateCommandNameError = DuplicateCommandNameError;
class DuplicateCommandAliasError extends CommandError {
    constructor(alias) {
        super(`Duplicate command alias "${alias}".`);
        Object.setPrototypeOf(this, DuplicateCommandAliasError.prototype);
    }
}
exports.DuplicateCommandAliasError = DuplicateCommandAliasError;
class CommandNotFoundError extends CommandError {
    constructor(name, commands, excluded) {
        super(`Unknown command "${name}".${(0, _utils_js_1.didYouMeanCommand)(name, commands, excluded)}`);
        Object.setPrototypeOf(this, CommandNotFoundError.prototype);
    }
}
exports.CommandNotFoundError = CommandNotFoundError;
class DuplicateTypeError extends CommandError {
    constructor(name) {
        super(`Type with name "${name}" already exists.`);
        Object.setPrototypeOf(this, DuplicateTypeError.prototype);
    }
}
exports.DuplicateTypeError = DuplicateTypeError;
class DuplicateCompletionError extends CommandError {
    constructor(name) {
        super(`Completion with name "${name}" already exists.`);
        Object.setPrototypeOf(this, DuplicateCompletionError.prototype);
    }
}
exports.DuplicateCompletionError = DuplicateCompletionError;
class DuplicateExampleError extends CommandError {
    constructor(name) {
        super(`Example with name "${name}" already exists.`);
        Object.setPrototypeOf(this, DuplicateExampleError.prototype);
    }
}
exports.DuplicateExampleError = DuplicateExampleError;
class DuplicateEnvVarError extends CommandError {
    constructor(name) {
        super(`Environment variable with name "${name}" already exists.`);
        Object.setPrototypeOf(this, DuplicateEnvVarError.prototype);
    }
}
exports.DuplicateEnvVarError = DuplicateEnvVarError;
class MissingRequiredEnvVarError extends ValidationError {
    constructor(envVar) {
        super(`Missing required environment variable "${envVar.names[0]}".`);
        Object.setPrototypeOf(this, MissingRequiredEnvVarError.prototype);
    }
}
exports.MissingRequiredEnvVarError = MissingRequiredEnvVarError;
class TooManyEnvVarValuesError extends CommandError {
    constructor(name) {
        super(`An environment variable can only have one value, but "${name}" has more than one.`);
        Object.setPrototypeOf(this, TooManyEnvVarValuesError.prototype);
    }
}
exports.TooManyEnvVarValuesError = TooManyEnvVarValuesError;
class UnexpectedOptionalEnvVarValueError extends CommandError {
    constructor(name) {
        super(`An environment variable cannot have an optional value, but "${name}" is defined as optional.`);
        Object.setPrototypeOf(this, UnexpectedOptionalEnvVarValueError.prototype);
    }
}
exports.UnexpectedOptionalEnvVarValueError = UnexpectedOptionalEnvVarValueError;
class UnexpectedVariadicEnvVarValueError extends CommandError {
    constructor(name) {
        super(`An environment variable cannot have an variadic value, but "${name}" is defined as variadic.`);
        Object.setPrototypeOf(this, UnexpectedVariadicEnvVarValueError.prototype);
    }
}
exports.UnexpectedVariadicEnvVarValueError = UnexpectedVariadicEnvVarValueError;
class DefaultCommandNotFoundError extends CommandError {
    constructor(name, commands) {
        super(`Default command "${name}" not found.${(0, _utils_js_1.didYouMeanCommand)(name, commands)}`);
        Object.setPrototypeOf(this, DefaultCommandNotFoundError.prototype);
    }
}
exports.DefaultCommandNotFoundError = DefaultCommandNotFoundError;
class CommandExecutableNotFoundError extends CommandError {
    constructor(name) {
        super(`Command executable not found: ${name}`);
        Object.setPrototypeOf(this, CommandExecutableNotFoundError.prototype);
    }
}
exports.CommandExecutableNotFoundError = CommandExecutableNotFoundError;
class UnknownCompletionCommandError extends CommandError {
    constructor(name, commands) {
        super(`Auto-completion failed. Unknown command "${name}".${(0, _utils_js_1.didYouMeanCommand)(name, commands)}`);
        Object.setPrototypeOf(this, UnknownCompletionCommandError.prototype);
    }
}
exports.UnknownCompletionCommandError = UnknownCompletionCommandError;
/* Validation errors. */
class UnknownCommandError extends ValidationError {
    constructor(name, commands, excluded) {
        super(`Unknown command "${name}".${(0, _utils_js_1.didYouMeanCommand)(name, commands, excluded)}`);
        Object.setPrototypeOf(this, UnknownCommandError.prototype);
    }
}
exports.UnknownCommandError = UnknownCommandError;
class NoArgumentsAllowedError extends ValidationError {
    constructor(name) {
        super(`No arguments allowed for command "${name}".`);
        Object.setPrototypeOf(this, NoArgumentsAllowedError.prototype);
    }
}
exports.NoArgumentsAllowedError = NoArgumentsAllowedError;
class MissingArgumentsError extends ValidationError {
    constructor(names) {
        super(`Missing argument(s): ${names.join(", ")}`);
        Object.setPrototypeOf(this, MissingArgumentsError.prototype);
    }
}
exports.MissingArgumentsError = MissingArgumentsError;
class MissingArgumentError extends ValidationError {
    constructor(name) {
        super(`Missing argument: ${name}`);
        Object.setPrototypeOf(this, MissingArgumentError.prototype);
    }
}
exports.MissingArgumentError = MissingArgumentError;
class TooManyArgumentsError extends ValidationError {
    constructor(args) {
        super(`Too many arguments: ${args.join(" ")}`);
        Object.setPrototypeOf(this, TooManyArgumentsError.prototype);
    }
}
exports.TooManyArgumentsError = TooManyArgumentsError;
