"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HelpGenerator = void 0;
const dntShim = __importStar(require("../../../../../../_dnt.shims.js"));
const _utils_js_1 = require("../../flags/_utils.js");
const table_js_1 = require("../../table/table.js");
const _utils_js_2 = require("../_utils.js");
const deps_js_1 = require("../deps.js");
const type_js_1 = require("../type.js");
/** Help text generator. */
class HelpGenerator {
    /** Generate help text for given command. */
    static generate(cmd, options) {
        return new HelpGenerator(cmd, options).generate();
    }
    constructor(cmd, options = {}) {
        Object.defineProperty(this, "cmd", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: cmd
        });
        Object.defineProperty(this, "indent", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: 2
        });
        Object.defineProperty(this, "options", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.options = {
            types: false,
            hints: true,
            colors: true,
            long: false,
            ...options,
        };
    }
    generate() {
        const areColorsEnabled = (0, deps_js_1.getColorEnabled)();
        (0, deps_js_1.setColorEnabled)(this.options.colors);
        const result = this.generateHeader() +
            this.generateMeta() +
            this.generateDescription() +
            this.generateOptions() +
            this.generateCommands() +
            this.generateEnvironmentVariables() +
            this.generateExamples();
        (0, deps_js_1.setColorEnabled)(areColorsEnabled);
        return result;
    }
    generateHeader() {
        const usage = this.cmd.getUsage();
        const rows = [
            [
                (0, deps_js_1.bold)("Usage:"),
                (0, deps_js_1.brightMagenta)(this.cmd.getPath() +
                    (usage ? " " + highlightArguments(usage, this.options.types) : "")),
            ],
        ];
        const version = this.cmd.getVersion();
        if (version) {
            rows.push([(0, deps_js_1.bold)("Version:"), (0, deps_js_1.yellow)(`${this.cmd.getVersion()}`)]);
        }
        return "\n" +
            table_js_1.Table.from(rows)
                .indent(this.indent)
                .padding(1)
                .toString() +
            "\n";
    }
    generateMeta() {
        const meta = Object.entries(this.cmd.getMeta());
        if (!meta.length) {
            return "";
        }
        const rows = [];
        for (const [name, value] of meta) {
            rows.push([(0, deps_js_1.bold)(`${name}: `) + value]);
        }
        return "\n" +
            table_js_1.Table.from(rows)
                .indent(this.indent)
                .padding(1)
                .toString() +
            "\n";
    }
    generateDescription() {
        if (!this.cmd.getDescription()) {
            return "";
        }
        return this.label("Description") +
            table_js_1.Table.from([
                [(0, _utils_js_2.dedent)(this.cmd.getDescription())],
            ])
                .indent(this.indent * 2)
                .maxColWidth(140)
                .padding(1)
                .toString() +
            "\n";
    }
    generateOptions() {
        const options = this.cmd.getOptions(false);
        if (!options.length) {
            return "";
        }
        let groups = [];
        const hasGroups = options.some((option) => option.groupName);
        if (hasGroups) {
            for (const option of options) {
                let group = groups.find((group) => group.name === option.groupName);
                if (!group) {
                    group = {
                        name: option.groupName,
                        options: [],
                    };
                    groups.push(group);
                }
                group.options.push(option);
            }
        }
        else {
            groups = [{
                    name: "Options",
                    options,
                }];
        }
        let result = "";
        for (const group of groups) {
            result += this.generateOptionGroup(group);
        }
        return result;
    }
    generateOptionGroup(group) {
        if (!group.options.length) {
            return "";
        }
        const hasTypeDefinitions = !!group.options.find((option) => !!option.typeDefinition);
        if (hasTypeDefinitions) {
            return this.label(group.name ?? "Options") +
                table_js_1.Table.from([
                    ...group.options.map((option) => [
                        option.flags.map((flag) => (0, deps_js_1.brightBlue)(flag)).join(", "),
                        highlightArguments(option.typeDefinition || "", this.options.types),
                        (0, deps_js_1.red)((0, deps_js_1.bold)("-")),
                        (0, _utils_js_2.getDescription)(option.description, !this.options.long),
                        this.generateHints(option),
                    ]),
                ])
                    .padding([2, 2, 1, 2])
                    .indent(this.indent * 2)
                    .maxColWidth([60, 60, 1, 80, 60])
                    .toString() +
                "\n";
        }
        return this.label(group.name ?? "Options") +
            table_js_1.Table.from([
                ...group.options.map((option) => [
                    option.flags.map((flag) => (0, deps_js_1.brightBlue)(flag)).join(", "),
                    (0, deps_js_1.red)((0, deps_js_1.bold)("-")),
                    (0, _utils_js_2.getDescription)(option.description, !this.options.long),
                    this.generateHints(option),
                ]),
            ])
                .indent(this.indent * 2)
                .maxColWidth([60, 1, 80, 60])
                .padding([2, 1, 2])
                .toString() +
            "\n";
    }
    generateCommands() {
        const commands = this.cmd.getCommands(false);
        if (!commands.length) {
            return "";
        }
        const hasTypeDefinitions = !!commands.find((command) => !!command.getArgsDefinition());
        if (hasTypeDefinitions) {
            return this.label("Commands") +
                table_js_1.Table.from([
                    ...commands.map((command) => [
                        [command.getName(), ...command.getAliases()].map((name) => (0, deps_js_1.brightBlue)(name)).join(", "),
                        highlightArguments(command.getArgsDefinition() || "", this.options.types),
                        (0, deps_js_1.red)((0, deps_js_1.bold)("-")),
                        command.getShortDescription(),
                    ]),
                ])
                    .indent(this.indent * 2)
                    .maxColWidth([60, 60, 1, 80])
                    .padding([2, 2, 1, 2])
                    .toString() +
                "\n";
        }
        return this.label("Commands") +
            table_js_1.Table.from([
                ...commands.map((command) => [
                    [command.getName(), ...command.getAliases()].map((name) => (0, deps_js_1.brightBlue)(name))
                        .join(", "),
                    (0, deps_js_1.red)((0, deps_js_1.bold)("-")),
                    command.getShortDescription(),
                ]),
            ])
                .maxColWidth([60, 1, 80])
                .padding([2, 1, 2])
                .indent(this.indent * 2)
                .toString() +
            "\n";
    }
    generateEnvironmentVariables() {
        const envVars = this.cmd.getEnvVars(false);
        if (!envVars.length) {
            return "";
        }
        return this.label("Environment variables") +
            table_js_1.Table.from([
                ...envVars.map((envVar) => [
                    envVar.names.map((name) => (0, deps_js_1.brightBlue)(name)).join(", "),
                    highlightArgumentDetails(envVar.details, this.options.types),
                    (0, deps_js_1.red)((0, deps_js_1.bold)("-")),
                    this.options.long
                        ? (0, _utils_js_2.dedent)(envVar.description)
                        : envVar.description.trim().split("\n", 1)[0],
                    envVar.required ? `(${(0, deps_js_1.yellow)(`required`)})` : "",
                ]),
            ])
                .padding([2, 2, 1, 2])
                .indent(this.indent * 2)
                .maxColWidth([60, 60, 1, 80, 10])
                .toString() +
            "\n";
    }
    generateExamples() {
        const examples = this.cmd.getExamples();
        if (!examples.length) {
            return "";
        }
        return this.label("Examples") +
            table_js_1.Table.from(examples.map((example) => [
                (0, deps_js_1.dim)((0, deps_js_1.bold)(`${capitalize(example.name)}:`)),
                (0, _utils_js_2.dedent)(example.description),
            ]))
                .padding(1)
                .indent(this.indent * 2)
                .maxColWidth(150)
                .toString() +
            "\n";
    }
    generateHints(option) {
        if (!this.options.hints) {
            return "";
        }
        const hints = [];
        option.required && hints.push((0, deps_js_1.yellow)(`required`));
        typeof option.default !== "undefined" && hints.push((0, deps_js_1.bold)(`Default: `) + inspect(option.default, this.options.colors));
        option.depends?.length && hints.push((0, deps_js_1.yellow)((0, deps_js_1.bold)(`Depends: `)) +
            (0, deps_js_1.italic)(option.depends.map(_utils_js_1.getFlag).join(", ")));
        option.conflicts?.length && hints.push((0, deps_js_1.red)((0, deps_js_1.bold)(`Conflicts: `)) +
            (0, deps_js_1.italic)(option.conflicts.map(_utils_js_1.getFlag).join(", ")));
        const type = this.cmd.getType(option.args[0]?.type)?.handler;
        if (type instanceof type_js_1.Type) {
            const possibleValues = type.values?.(this.cmd, this.cmd.getParent());
            if (possibleValues?.length) {
                hints.push((0, deps_js_1.bold)(`Values: `) +
                    possibleValues.map((value) => inspect(value, this.options.colors)).join(", "));
            }
        }
        if (hints.length) {
            return `(${hints.join(", ")})`;
        }
        return "";
    }
    label(label) {
        return "\n" +
            " ".repeat(this.indent) + (0, deps_js_1.bold)(`${label}:`) +
            "\n\n";
    }
}
exports.HelpGenerator = HelpGenerator;
function capitalize(string) {
    return string?.charAt(0).toUpperCase() + string.slice(1) ?? "";
}
function inspect(value, colors) {
    return dntShim.Deno.inspect(value, 
    // deno < 1.4.3 doesn't support the colors property.
    { depth: 1, colors, trailingComma: false });
}
/**
 * Colorize arguments string.
 * @param argsDefinition Arguments definition: `<color1:string> <color2:string>`
 * @param types Show types.
 */
function highlightArguments(argsDefinition, types = true) {
    if (!argsDefinition) {
        return "";
    }
    return (0, _utils_js_2.parseArgumentsDefinition)(argsDefinition, false, true)
        .map((arg) => typeof arg === "string" ? arg : highlightArgumentDetails(arg, types))
        .join(" ");
}
/**
 * Colorize argument string.
 * @param arg Argument details.
 * @param types Show types.
 */
function highlightArgumentDetails(arg, types = true) {
    let str = "";
    str += (0, deps_js_1.yellow)(arg.optionalValue ? "[" : "<");
    let name = "";
    name += arg.name;
    if (arg.variadic) {
        name += "...";
    }
    name = (0, deps_js_1.brightMagenta)(name);
    str += name;
    if (types) {
        str += (0, deps_js_1.yellow)(":");
        str += (0, deps_js_1.red)(arg.type);
        if (arg.list) {
            str += (0, deps_js_1.green)("[]");
        }
    }
    str += (0, deps_js_1.yellow)(arg.optionalValue ? "]" : ">");
    return str;
}
