"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultValue = exports.matchWildCardOptions = exports.getFlag = exports.didYouMean = exports.didYouMeanType = exports.didYouMeanOption = exports.getOption = exports.underscoreToCamelCase = exports.paramCaseToCamelCase = void 0;
const distance_js_1 = require("../_utils/distance.js");
/** Convert param case string to camel case. */
function paramCaseToCamelCase(str) {
    return str.replace(/-([a-z])/g, (g) => g[1].toUpperCase());
}
exports.paramCaseToCamelCase = paramCaseToCamelCase;
/** Convert underscore case string to camel case. */
function underscoreToCamelCase(str) {
    return str
        .replace(/([a-z])([A-Z])/g, "$1_$2")
        .toLowerCase()
        .replace(/_([a-z])/g, (g) => g[1].toUpperCase());
}
exports.underscoreToCamelCase = underscoreToCamelCase;
/**
 * Find option by flag, name or alias.
 *
 * @param flags Source options array.
 * @param name  Name of the option.
 */
function getOption(flags, name) {
    while (name[0] === "-") {
        name = name.slice(1);
    }
    for (const flag of flags) {
        if (isOption(flag, name)) {
            return flag;
        }
    }
    return;
}
exports.getOption = getOption;
function didYouMeanOption(option, options) {
    const optionNames = options
        .map((option) => [option.name, ...(option.aliases ?? [])])
        .flat()
        .map((option) => getFlag(option));
    return didYouMean(" Did you mean option", getFlag(option), optionNames);
}
exports.didYouMeanOption = didYouMeanOption;
function didYouMeanType(type, types) {
    return didYouMean(" Did you mean type", type, types);
}
exports.didYouMeanType = didYouMeanType;
function didYouMean(message, type, types) {
    const match = closest(type, types);
    return match ? `${message} "${match}"?` : "";
}
exports.didYouMean = didYouMean;
function getFlag(name) {
    if (name.startsWith("-")) {
        return name;
    }
    if (name.length > 1) {
        return `--${name}`;
    }
    return `-${name}`;
}
exports.getFlag = getFlag;
/**
 * Check if option has name or alias.
 *
 * @param option    The option to check.
 * @param name      The option name or alias.
 */
function isOption(option, name) {
    return option.name === name ||
        (option.aliases && option.aliases.indexOf(name) !== -1);
}
function matchWildCardOptions(name, flags) {
    for (const option of flags) {
        if (option.name.indexOf("*") === -1) {
            continue;
        }
        let matched = matchWildCardOption(name, option);
        if (matched) {
            matched = { ...matched, name };
            flags.push(matched);
            return matched;
        }
    }
}
exports.matchWildCardOptions = matchWildCardOptions;
function matchWildCardOption(name, option) {
    const parts = option.name.split(".");
    const parts2 = name.split(".");
    if (parts.length !== parts2.length) {
        return false;
    }
    const count = Math.max(parts.length, parts2.length);
    for (let i = 0; i < count; i++) {
        if (parts[i] !== parts2[i] && parts[i] !== "*") {
            return false;
        }
    }
    return option;
}
function closest(str, arr) {
    let minDistance = Infinity;
    let minIndex = 0;
    for (let i = 0; i < arr.length; i++) {
        const dist = (0, distance_js_1.distance)(str, arr[i]);
        if (dist < minDistance) {
            minDistance = dist;
            minIndex = i;
        }
    }
    return arr[minIndex];
}
function getDefaultValue(option) {
    return typeof option.default === "function"
        ? option.default()
        : option.default;
}
exports.getDefaultValue = getDefaultValue;
