"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.strLength = exports.longest = exports.consumeWords = void 0;
/**
 * Get next words from the beginning of [content] until all words have a length lower or equal then [length].
 *
 * @param length    Max length of all words.
 * @param content   The text content.
 */
const cell_js_1 = require("./cell.js");
const deps_js_1 = require("./deps.js");
function consumeWords(length, content) {
    let consumed = "";
    const words = content.split("\n")[0]?.split(/ /g);
    for (let i = 0; i < words.length; i++) {
        const word = words[i];
        // consume minimum one word
        if (consumed) {
            const nextLength = (0, exports.strLength)(word);
            const consumedLength = (0, exports.strLength)(consumed);
            if (consumedLength + nextLength >= length) {
                break;
            }
        }
        consumed += (i > 0 ? " " : "") + word;
    }
    return consumed;
}
exports.consumeWords = consumeWords;
/**
 * Get longest cell from given row index.
 */
function longest(index, rows, maxWidth) {
    const cellLengths = rows.map((row) => {
        const cell = row[index];
        const cellValue = cell instanceof cell_js_1.Cell && cell.getColSpan() > 1
            ? ""
            : cell?.toString() || "";
        return cellValue
            .split("\n")
            .map((line) => {
            const str = typeof maxWidth === "undefined"
                ? line
                : consumeWords(maxWidth, line);
            return (0, exports.strLength)(str) || 0;
        });
    }).flat();
    return Math.max(...cellLengths);
}
exports.longest = longest;
const strLength = (str) => {
    str = (0, deps_js_1.stripColor)(str);
    let length = 0;
    for (let i = 0; i < str.length; i++) {
        const charCode = str.charCodeAt(i);
        // Check for chinese characters: \u4e00 - \u9fa5
        if (charCode >= 19968 && charCode <= 40869) {
            length += 2;
        }
        else {
            length += 1;
        }
    }
    return length;
};
exports.strLength = strLength;
